#!/usr/bin/python3.10
# coding: utf-8
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved
"""dependencies"""

__all__ = ["get_db_ops", "get_db_agent"]

from openjiuwen_studio.ops.modules.prompt.infra.database import get_db_agent, get_db_ops