#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
"""schema updates for develop-20260116

Revision ID: 13377a900fe2
Revises: 80f110f929fc
Create Date: 2026-01-20 21:08:28.719074

"""

from typing import Sequence, Union
from openjiuwen_studio.core.database.migration_utils import table_exists, column_exists, index_exists



# revision identifiers, used by Alembic.
revision: str = "13377a900fe2"
down_revision: Union[str, None] = "80f110f929fc"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
