"""update_sqlite_ops_v0.1.3_20260126

Revision ID: f6e49cd8c97d
Revises: b4f4c6589bc5
Create Date: 2026-01-26 20:56:12.372693

"""
from typing import Sequence, Union

from alembic import op
from openjiuwen_studio.core.database.migration_utils import table_exists, column_exists, index_exists

import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = 'f6e49cd8c97d'
down_revision: Union[str, Sequence[str], None] = 'b4f4c6589bc5'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###